@@ -1,17 +1,21 @@
-from __future__ import absolute_import, unicode_literals
-
 from collections import namedtuple
 from datetime import datetime, time
 
-import django
 from django import forms
 from django.utils.dateparse import parse_datetime
 from django.utils.encoding import force_str
 from django.utils.translation import ugettext_lazy as _
 
 from .conf import settings
+from .constants import EMPTY_VALUES
 from .utils import handle_timezone
-from .widgets import BaseCSVWidget, CSVWidget, LookupTypeWidget, RangeWidget
+from .widgets import (
+    BaseCSVWidget,
+    CSVWidget,
+    DateRangeWidget,
+    LookupChoiceWidget,
+    RangeWidget
+)
 
 
 class RangeField(forms.MultiValueField):
@@ -22,7 +26,7 @@ class RangeField(forms.MultiValueField):
             fields = (
                 forms.DecimalField(),
                 forms.DecimalField())
-        super(RangeField, self).__init__(fields, *args, **kwargs)
+        super().__init__(fields, *args, **kwargs)
 
     def compress(self, data_list):
         if data_list:
@@ -31,12 +35,13 @@ class RangeField(forms.MultiValueField):
 
 
 class DateRangeField(RangeField):
+    widget = DateRangeWidget
 
     def __init__(self, *args, **kwargs):
         fields = (
             forms.DateField(),
             forms.DateField())
-        super(DateRangeField, self).__init__(fields, *args, **kwargs)
+        super().__init__(fields, *args, **kwargs)
 
     def compress(self, data_list):
         if data_list:
@@ -56,49 +61,70 @@ class DateRangeField(RangeField):
 
 
 class DateTimeRangeField(RangeField):
+    widget = DateRangeWidget
 
     def __init__(self, *args, **kwargs):
         fields = (
             forms.DateTimeField(),
             forms.DateTimeField())
-        super(DateTimeRangeField, self).__init__(fields, *args, **kwargs)
+        super().__init__(fields, *args, **kwargs)
+
+
+class IsoDateTimeRangeField(RangeField):
+    widget = DateRangeWidget
+
+    def __init__(self, *args, **kwargs):
+        fields = (
+            IsoDateTimeField(),
+            IsoDateTimeField())
+        super().__init__(fields, *args, **kwargs)
 
 
 class TimeRangeField(RangeField):
+    widget = DateRangeWidget
 
     def __init__(self, *args, **kwargs):
         fields = (
             forms.TimeField(),
             forms.TimeField())
-        super(TimeRangeField, self).__init__(fields, *args, **kwargs)
+        super().__init__(fields, *args, **kwargs)
 
 
-class Lookup(namedtuple('Lookup', ('value', 'lookup_type'))):
-    # python nature is test __len__ on tuple types for boolean check
-    def __len__(self):
-        if not self.value:
-            return 0
-        return 2
+class Lookup(namedtuple('Lookup', ('value', 'lookup_expr'))):
+    def __new__(cls, value, lookup_expr):
+        if value in EMPTY_VALUES or lookup_expr in EMPTY_VALUES:
+            raise ValueError(
+                "Empty values ([], (), {}, '', None) are not "
+                "valid Lookup arguments. Return None instead."
+            )
+
+        return super().__new__(cls, value, lookup_expr)
 
 
-class LookupTypeField(forms.MultiValueField):
+class LookupChoiceField(forms.MultiValueField):
+    default_error_messages = {
+        'lookup_required': _('Select a lookup.'),
+    }
+
     def __init__(self, field, lookup_choices, *args, **kwargs):
-        fields = (
-            field,
-            forms.ChoiceField(choices=lookup_choices)
-        )
-        defaults = {
-            'widgets': [f.widget for f in fields],
-        }
-        widget = LookupTypeWidget(**defaults)
+        empty_label = kwargs.pop('empty_label', settings.EMPTY_CHOICE_LABEL)
+        fields = (field, ChoiceField(choices=lookup_choices, empty_label=empty_label))
+        widget = LookupChoiceWidget(widgets=[f.widget for f in fields])
         kwargs['widget'] = widget
         kwargs['help_text'] = field.help_text
-        super(LookupTypeField, self).__init__(fields, *args, **kwargs)
+        super().__init__(fields, *args, **kwargs)
 
     def compress(self, data_list):
         if len(data_list) == 2:
-            return Lookup(value=data_list[0], lookup_type=data_list[1] or 'exact')
-        return Lookup(value=None, lookup_type='exact')
+            value, lookup_expr = data_list
+            if value not in EMPTY_VALUES:
+                if lookup_expr not in EMPTY_VALUES:
+                    return Lookup(value=value, lookup_expr=lookup_expr)
+                else:
+                    raise forms.ValidationError(
+                        self.error_messages['lookup_required'],
+                        code='lookup_required')
+        return None
 
 
 class IsoDateTimeField(forms.DateTimeField):
@@ -121,7 +147,7 @@ class IsoDateTimeField(forms.DateTimeField):
             if parsed is None:  # Continue with other formats if doesn't match
                 raise ValueError
             return handle_timezone(parsed)
-        return super(IsoDateTimeField, self).strptime(value, format)
+        return super().strptime(value, format)
 
 
 class BaseCSVField(forms.Field):
@@ -140,7 +166,7 @@ class BaseCSVField(forms.Field):
         widget = kwargs.get('widget') or self.widget
         kwargs['widget'] = self._get_widget_class(widget)
 
-        super(BaseCSVField, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def _get_widget_class(self, widget):
         # passthrough, allows for override
@@ -173,9 +199,11 @@ class BaseRangeField(BaseCSVField):
     }
 
     def clean(self, value):
-        value = super(BaseRangeField, self).clean(value)
+        value = super().clean(value)
 
-        if value is not None and len(value) != 2:
+        assert value is None or isinstance(value, list)
+
+        if value and len(value) != 2:
             raise forms.ValidationError(
                 self.error_messages['invalid_values'],
                 code='invalid_values')
@@ -213,7 +241,7 @@ class ModelChoiceIterator(forms.models.ModelChoiceIterator):
     # empty choice, but before the remainder of the choices.
 
     def __iter__(self):
-        iterable = super(ModelChoiceIterator, self).__iter__()
+        iterable = super().__iter__()
 
         if self.field.empty_label is not None:
             yield next(iterable)
@@ -226,7 +254,7 @@ class ModelChoiceIterator(forms.models.ModelChoiceIterator):
 
     def __len__(self):
         add = 1 if self.field.null_label is not None else 0
-        return super(ModelChoiceIterator, self).__len__() + add
+        return super().__len__() + add
 
 
 class ChoiceIteratorMixin(object):
@@ -234,20 +262,13 @@ class ChoiceIteratorMixin(object):
         self.null_label = kwargs.pop('null_label', settings.NULL_CHOICE_LABEL)
         self.null_value = kwargs.pop('null_value', settings.NULL_CHOICE_VALUE)
 
-        super(ChoiceIteratorMixin, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def _get_choices(self):
-        if django.VERSION >= (1, 11):
-            return super(ChoiceIteratorMixin, self)._get_choices()
-
-        # HACK: Django < 1.11 does not allow a custom iterator to be provided.
-        # This code only executes for Model*ChoiceFields.
-        if hasattr(self, '_choices'):
-            return self._choices
-        return self.iterator(self)
+        return super()._get_choices()
 
     def _set_choices(self, value):
-        super(ChoiceIteratorMixin, self)._set_choices(value)
+        super()._set_choices(value)
         value = self.iterator(self, self._choices)
 
         self._choices = self.widget.choices = value
@@ -260,7 +281,7 @@ class ChoiceField(ChoiceIteratorMixin, forms.ChoiceField):
 
     def __init__(self, *args, **kwargs):
         self.empty_label = kwargs.pop('empty_label', settings.EMPTY_CHOICE_LABEL)
-        super(ChoiceField, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
 
 class MultipleChoiceField(ChoiceIteratorMixin, forms.MultipleChoiceField):
@@ -268,7 +289,7 @@ class MultipleChoiceField(ChoiceIteratorMixin, forms.MultipleChoiceField):
 
     def __init__(self, *args, **kwargs):
         self.empty_label = None
-        super(MultipleChoiceField, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
 
 class ModelChoiceField(ChoiceIteratorMixin, forms.ModelChoiceField):
@@ -278,7 +299,7 @@ class ModelChoiceField(ChoiceIteratorMixin, forms.ModelChoiceField):
         # bypass the queryset value check
         if self.null_label is not None and value == self.null_value:
             return value
-        return super(ModelChoiceField, self).to_python(value)
+        return super().to_python(value)
 
 
 class ModelMultipleChoiceField(ChoiceIteratorMixin, forms.ModelMultipleChoiceField):
@@ -289,6 +310,6 @@ class ModelMultipleChoiceField(ChoiceIteratorMixin, forms.ModelMultipleChoiceFie
         if null:  # remove the null value and any potential duplicates
             value = [v for v in value if v != self.null_value]
 
-        result = list(super(ModelMultipleChoiceField, self)._check_values(value))
+        result = list(super()._check_values(value))
         result += [self.null_value] if null else []
         return result
