@@ -1,10 +1,7 @@
-from __future__ import absolute_import, unicode_literals
-
-from collections import Iterable
+from collections.abc import Iterable
 from itertools import chain
 from re import search, sub
 
-import django
 from django import forms
 from django.db.models.fields import BLANK_CHOICE_DASH
 from django.forms.utils import flatatt
@@ -12,32 +9,26 @@ from django.utils.datastructures import MultiValueDict
 from django.utils.encoding import force_text
 from django.utils.http import urlencode
 from django.utils.safestring import mark_safe
-from django.utils.six import string_types
 from django.utils.translation import ugettext as _
 
-from .compat import format_value
-
 
 class LinkWidget(forms.Widget):
     def __init__(self, attrs=None, choices=()):
-        super(LinkWidget, self).__init__(attrs)
+        super().__init__(attrs)
 
         self.choices = choices
 
     def value_from_datadict(self, data, files, name):
-        value = super(LinkWidget, self).value_from_datadict(data, files, name)
+        value = super().value_from_datadict(data, files, name)
         self.data = data
         return value
 
-    def render(self, name, value, attrs=None, choices=()):
+    def render(self, name, value, attrs=None, choices=(), renderer=None):
         if not hasattr(self, 'data'):
             self.data = {}
         if value is None:
             value = ''
-        if django.VERSION < (1, 11):
-            final_attrs = self.build_attrs(attrs)
-        else:
-            final_attrs = self.build_attrs(self.attrs, extra_attrs=attrs)
+        final_attrs = self.build_attrs(self.attrs, extra_attrs=attrs)
         output = ['<ul%s>' % flatatt(final_attrs)]
         options = self.render_options(choices, [value], name)
         if options:
@@ -91,7 +82,7 @@ class SuffixedMultiWidget(forms.MultiWidget):
     suffixes = []
 
     def __init__(self, *args, **kwargs):
-        super(SuffixedMultiWidget, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
         assert len(self.widgets) == len(self.suffixes)
         assert len(self.suffixes) == len(set(self.suffixes))
@@ -100,7 +91,7 @@ class SuffixedMultiWidget(forms.MultiWidget):
         return '_'.join([name, suffix]) if suffix else name
 
     def get_context(self, name, value, attrs):
-        context = super(SuffixedMultiWidget, self).get_context(name, value, attrs)
+        context = super().get_context(name, value, attrs)
         for subcontext, suffix in zip(context['widget']['subwidgets'], self.suffixes):
             subcontext['name'] = self.suffixed(name, suffix)
 
@@ -118,14 +109,6 @@ class SuffixedMultiWidget(forms.MultiWidget):
             for widget, suffix in zip(self.widgets, self.suffixes)
         )
 
-    # Django < 1.11 compat
-    def format_output(self, rendered_widgets):
-        rendered_widgets = [
-            self.replace_name(output, i)
-            for i, output in enumerate(rendered_widgets)
-        ]
-        return '\n'.join(rendered_widgets)
-
     def replace_name(self, output, index):
         result = search(r'name="(?P<name>.*)_%d"' % index, output)
         name = result.group('name')
@@ -140,16 +123,13 @@ class SuffixedMultiWidget(forms.MultiWidget):
         return value
 
 
-class RangeWidget(forms.MultiWidget):
+class RangeWidget(SuffixedMultiWidget):
     template_name = 'django_filters/widgets/multiwidget.html'
+    suffixes = ['min', 'max']
 
     def __init__(self, attrs=None):
         widgets = (forms.TextInput, forms.TextInput)
-        super(RangeWidget, self).__init__(widgets, attrs)
-
-    def format_output(self, rendered_widgets):
-        # Method was removed in Django 1.11.
-        return '-'.join(rendered_widgets)
+        super().__init__(widgets, attrs)
 
     def decompress(self, value):
         if value:
@@ -157,7 +137,13 @@ class RangeWidget(forms.MultiWidget):
         return [None, None]
 
 
-class LookupTypeWidget(forms.MultiWidget):
+class DateRangeWidget(RangeWidget):
+    suffixes = ['after', 'before']
+
+
+class LookupChoiceWidget(SuffixedMultiWidget):
+    suffixes = [None, 'lookup']
+
     def decompress(self, value):
         if value is None:
             return [None, None]
@@ -173,9 +159,9 @@ class BooleanWidget(forms.Select):
         choices = (('', _('Unknown')),
                    ('true', _('Yes')),
                    ('false', _('No')))
-        super(BooleanWidget, self).__init__(attrs, choices)
+        super().__init__(attrs, choices)
 
-    def render(self, name, value, attrs=None):
+    def render(self, name, value, attrs=None, renderer=None):
         try:
             value = {
                 True: 'true',
@@ -185,11 +171,11 @@ class BooleanWidget(forms.Select):
             }[value]
         except KeyError:
             value = ''
-        return super(BooleanWidget, self).render(name, value, attrs)
+        return super().render(name, value, attrs, renderer=renderer)
 
     def value_from_datadict(self, data, files, name):
         value = data.get(name, None)
-        if isinstance(value, string_types):
+        if isinstance(value, str):
             value = value.lower()
 
         return {
@@ -204,10 +190,10 @@ class BooleanWidget(forms.Select):
 
 class BaseCSVWidget(forms.Widget):
     def _isiterable(self, value):
-        return isinstance(value, Iterable) and not isinstance(value, string_types)
+        return isinstance(value, Iterable) and not isinstance(value, str)
 
     def value_from_datadict(self, data, files, name):
-        value = super(BaseCSVWidget, self).value_from_datadict(data, files, name)
+        value = super().value_from_datadict(data, files, name)
 
         if value is not None:
             if value == '':  # empty value should parse as an empty list
@@ -215,22 +201,22 @@ class BaseCSVWidget(forms.Widget):
             return value.split(',')
         return None
 
-    def render(self, name, value, attrs=None):
+    def render(self, name, value, attrs=None, renderer=None):
         if not self._isiterable(value):
             value = [value]
 
         if len(value) <= 1:
             # delegate to main widget (Select, etc...) if not multiple values
             value = value[0] if value else ''
-            return super(BaseCSVWidget, self).render(name, value, attrs)
+            return super().render(name, value, attrs, renderer=renderer)
 
         # if we have multiple values, we need to force render as a text input
         # (otherwise, the additional values are lost)
         surrogate = forms.TextInput()
-        value = [force_text(format_value(surrogate, v)) for v in value]
+        value = [force_text(surrogate.format_value(v)) for v in value]
         value = ','.join(list(value))
 
-        return surrogate.render(name, value, attrs)
+        return surrogate.render(name, value, attrs, renderer=renderer)
 
 
 class CSVWidget(BaseCSVWidget, forms.TextInput):
@@ -252,7 +238,7 @@ class QueryArrayWidget(BaseCSVWidget, forms.TextInput):
         if not isinstance(data, MultiValueDict):
             for key, value in data.items():
                 # treat value as csv string: ?foo=1,2
-                if isinstance(value, string_types):
+                if isinstance(value, str):
                     data[key] = [x.strip() for x in value.rstrip(',').split(',') if x]
             data = MultiValueDict(data)
 
