@@ -1,6 +1,4 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import, unicode_literals
-
 import inspect
 import mock
 from collections import OrderedDict
@@ -16,13 +14,12 @@ from django_filters.fields import (
     BaseCSVField,
     DateRangeField,
     DateTimeRangeField,
+    IsoDateTimeRangeField,
     Lookup,
-    LookupTypeField,
     RangeField,
     TimeRangeField
 )
 from django_filters.filters import (
-    LOOKUP_TYPES,
     AllValuesFilter,
     BaseCSVFilter,
     BaseInFilter,
@@ -37,6 +34,8 @@ from django_filters.filters import (
     DateTimeFromToRangeFilter,
     DurationFilter,
     Filter,
+    IsoDateTimeFromToRangeFilter,
+    LookupChoiceFilter,
     ModelChoiceFilter,
     ModelMultipleChoiceFilter,
     MultipleChoiceFilter,
@@ -96,29 +95,23 @@ class FilterTests(TestCase):
         field = f.field
         self.assertIsInstance(field, forms.Field)
 
-    def test_field_with_none_lookup_expr(self):
-        f = Filter(lookup_expr=None)
-        field = f.field
-        self.assertIsInstance(field, LookupTypeField)
-        choice_field = field.fields[1]
-        self.assertEqual(len(choice_field.choices), len(LOOKUP_TYPES))
+    def test_field_with_lookup_types_removal(self):
+        msg = (
+            "The `lookup_expr` argument no longer accepts `None` or a list of "
+            "expressions. Use the `LookupChoiceFilter` instead. See: "
+            "https://django-filter.readthedocs.io/en/master/guide/migration.html"
+        )
 
-    def test_field_with_lookup_expr_and_exlusion(self):
-        f = Filter(lookup_expr=None, exclude=True)
-        field = f.field
-        self.assertIsInstance(field, LookupTypeField)
+        with self.assertRaisesMessage(AssertionError, msg):
+            Filter(lookup_expr=[])
 
-    def test_field_with_list_lookup_expr(self):
-        f = Filter(lookup_expr=('istartswith', 'iendswith'))
-        field = f.field
-        self.assertIsInstance(field, LookupTypeField)
-        choice_field = field.fields[1]
-        self.assertEqual(len(choice_field.choices), 2)
+        with self.assertRaisesMessage(AssertionError, msg):
+            Filter(lookup_expr=None)
 
     def test_field_params(self):
         with mock.patch.object(Filter, 'field_class',
                                spec=['__call__']) as mocked:
-            f = Filter(name='somefield', label='somelabel',
+            f = Filter(field_name='somefield', label='somelabel',
                        widget='somewidget')
             f.field
             mocked.assert_called_once_with(required=False,
@@ -159,7 +152,7 @@ class FilterTests(TestCase):
 
     def test_filtering_uses_name(self):
         qs = mock.Mock(spec=['filter'])
-        f = Filter(name='somefield')
+        f = Filter(field_name='somefield')
         f.filter(qs, 'value')
         result = qs.filter.assert_called_once_with(somefield__exact='value')
         self.assertNotEqual(qs, result)
@@ -178,28 +171,6 @@ class FilterTests(TestCase):
         self.assertListEqual(qs.method_calls, [])
         self.assertEqual(qs, result)
 
-    def test_filtering_with_list_value(self):
-        qs = mock.Mock(spec=['filter'])
-        f = Filter(name='somefield', lookup_expr=['some_lookup_expr'])
-        result = f.filter(qs, Lookup('value', 'some_lookup_expr'))
-        qs.filter.assert_called_once_with(somefield__some_lookup_expr='value')
-        self.assertNotEqual(qs, result)
-
-    def test_filtering_skipped_with_list_value_with_blank(self):
-        qs = mock.Mock()
-        f = Filter(name='somefield', lookup_expr=['some_lookup_expr'])
-        result = f.filter(qs, Lookup('', 'some_lookup_expr'))
-        self.assertListEqual(qs.method_calls, [])
-        self.assertEqual(qs, result)
-
-    def test_filtering_skipped_with_list_value_with_blank_lookup(self):
-        return  # Now field is required to provide valid lookup_expr if it provides any
-        qs = mock.Mock(spec=['filter'])
-        f = Filter(name='somefield', lookup_expr=None)
-        result = f.filter(qs, Lookup('value', ''))
-        qs.filter.assert_called_once_with(somefield__exact='value')
-        self.assertNotEqual(qs, result)
-
     def test_filter_using_method(self):
         qs = mock.NonCallableMock(spec=[])
         method = mock.Mock()
@@ -210,39 +181,12 @@ class FilterTests(TestCase):
 
     def test_filtering_uses_distinct(self):
         qs = mock.Mock(spec=['filter', 'distinct'])
-        f = Filter(name='somefield', distinct=True)
+        f = Filter(field_name='somefield', distinct=True)
         f.filter(qs, 'value')
         result = qs.distinct.assert_called_once_with()
         self.assertNotEqual(qs, result)
 
 
-class CustomFilterWithBooleanCheckTests(TestCase):
-
-    def setUp(self):
-        super(CustomFilterWithBooleanCheckTests, self).setUp()
-
-        class CustomTestFilter(Filter):
-            def filter(self_, qs, value):
-                if not value:
-                    return qs
-                return super(CustomTestFilter, self_).filter(qs, value)
-
-        self.test_filter_class = CustomTestFilter
-
-    def test_lookup_false(self):
-        qs = mock.Mock(spec=['filter'])
-        f = self.test_filter_class(name='somefield')
-        result = f.filter(qs, Lookup('', 'exact'))
-        self.assertEqual(qs, result)
-
-    def test_lookup_true(self):
-        qs = mock.Mock(spec=['filter'])
-        f = self.test_filter_class(name='somefield')
-        result = f.filter(qs, Lookup('somesearch', 'exact'))
-        qs.filter.assert_called_once_with(somefield__exact='somesearch')
-        self.assertNotEqual(qs, result)
-
-
 class CharFilterTests(TestCase):
 
     def test_default_field(self):
@@ -268,35 +212,35 @@ class BooleanFilterTests(TestCase):
 
     def test_filtering(self):
         qs = mock.Mock(spec=['filter'])
-        f = BooleanFilter(name='somefield')
+        f = BooleanFilter(field_name='somefield')
         result = f.filter(qs, True)
         qs.filter.assert_called_once_with(somefield__exact=True)
         self.assertNotEqual(qs, result)
 
     def test_filtering_exclude(self):
         qs = mock.Mock(spec=['exclude'])
-        f = BooleanFilter(name='somefield', exclude=True)
+        f = BooleanFilter(field_name='somefield', exclude=True)
         result = f.filter(qs, True)
         qs.exclude.assert_called_once_with(somefield__exact=True)
         self.assertNotEqual(qs, result)
 
     def test_filtering_skipped_with_blank_value(self):
         qs = mock.Mock()
-        f = BooleanFilter(name='somefield')
+        f = BooleanFilter(field_name='somefield')
         result = f.filter(qs, '')
         self.assertListEqual(qs.method_calls, [])
         self.assertEqual(qs, result)
 
     def test_filtering_skipped_with_none_value(self):
         qs = mock.Mock()
-        f = BooleanFilter(name='somefield')
+        f = BooleanFilter(field_name='somefield')
         result = f.filter(qs, None)
         self.assertListEqual(qs.method_calls, [])
         self.assertEqual(qs, result)
 
     def test_filtering_lookup_expr(self):
         qs = mock.Mock(spec=['filter'])
-        f = BooleanFilter(name='somefield', lookup_expr='isnull')
+        f = BooleanFilter(field_name='somefield', lookup_expr='isnull')
         result = f.filter(qs, True)
         qs.filter.assert_called_once_with(somefield__isnull=True)
         self.assertNotEqual(qs, result)
@@ -447,7 +391,7 @@ class MultipleChoiceFilterTests(TestCase):
 
     def test_filtering(self):
         qs = mock.Mock(spec=['filter'])
-        f = MultipleChoiceFilter(name='somefield')
+        f = MultipleChoiceFilter(field_name='somefield')
         with mock.patch('django_filters.filters.Q') as mockQclass:
             mockQ1, mockQ2 = mock.MagicMock(), mock.MagicMock()
             mockQclass.side_effect = [mockQ1, mockQ2]
@@ -462,7 +406,7 @@ class MultipleChoiceFilterTests(TestCase):
 
     def test_filtering_exclude(self):
         qs = mock.Mock(spec=['exclude'])
-        f = MultipleChoiceFilter(name='somefield', exclude=True)
+        f = MultipleChoiceFilter(field_name='somefield', exclude=True)
         with mock.patch('django_filters.filters.Q') as mockQclass:
             mockQ1, mockQ2 = mock.MagicMock(), mock.MagicMock()
             mockQclass.side_effect = [mockQ1, mockQ2]
@@ -477,7 +421,7 @@ class MultipleChoiceFilterTests(TestCase):
 
     def test_filtering_on_required_skipped_when_len_of_value_is_len_of_field_choices(self):
         qs = mock.Mock(spec=[])
-        f = MultipleChoiceFilter(name='somefield', required=True)
+        f = MultipleChoiceFilter(field_name='somefield', required=True)
         f.always_filter = False
         result = f.filter(qs, [])
         self.assertEqual(len(f.field.choices), 0)
@@ -492,7 +436,7 @@ class MultipleChoiceFilterTests(TestCase):
 
     def test_filtering_skipped_with_empty_list_value_and_some_choices(self):
         qs = mock.Mock(spec=[])
-        f = MultipleChoiceFilter(name='somefield')
+        f = MultipleChoiceFilter(field_name='somefield')
         f.field.choices = ['some', 'values', 'here']
         result = f.filter(qs, [])
         self.assertEqual(qs, result)
@@ -551,7 +495,7 @@ class MultipleChoiceFilterTests(TestCase):
         users = User.objects.all()
 
         for item in filter_list:
-            f = MultipleChoiceFilter(name='favorite_books__pk', conjoined=True)
+            f = MultipleChoiceFilter(field_name='favorite_books__pk', conjoined=True)
             queryset = f.filter(users, item[0])
             expected_pks = [c[0] for c in queryset.values_list('pk')]
             self.assertListEqual(
@@ -584,7 +528,7 @@ class TypedMultipleChoiceFilterTests(TestCase):
 
     def test_filtering(self):
         qs = mock.Mock(spec=['filter'])
-        f = TypedMultipleChoiceFilter(name='somefield')
+        f = TypedMultipleChoiceFilter(field_name='somefield')
         with mock.patch('django_filters.filters.Q') as mockQclass:
             mockQ1, mockQ2 = mock.MagicMock(), mock.MagicMock()
             mockQclass.side_effect = [mockQ1, mockQ2]
@@ -599,7 +543,7 @@ class TypedMultipleChoiceFilterTests(TestCase):
 
     def test_filtering_exclude(self):
         qs = mock.Mock(spec=['exclude'])
-        f = TypedMultipleChoiceFilter(name='somefield', exclude=True)
+        f = TypedMultipleChoiceFilter(field_name='somefield', exclude=True)
         with mock.patch('django_filters.filters.Q') as mockQclass:
             mockQ1, mockQ2 = mock.MagicMock(), mock.MagicMock()
             mockQclass.side_effect = [mockQ1, mockQ2]
@@ -614,7 +558,7 @@ class TypedMultipleChoiceFilterTests(TestCase):
 
     def test_filtering_on_required_skipped_when_len_of_value_is_len_of_field_choices(self):
         qs = mock.Mock(spec=[])
-        f = TypedMultipleChoiceFilter(name='somefield', required=True)
+        f = TypedMultipleChoiceFilter(field_name='somefield', required=True)
         f.always_filter = False
         result = f.filter(qs, [])
         self.assertEqual(len(f.field.choices), 0)
@@ -629,7 +573,7 @@ class TypedMultipleChoiceFilterTests(TestCase):
 
     def test_filtering_skipped_with_empty_list_value_and_some_choices(self):
         qs = mock.Mock(spec=[])
-        f = TypedMultipleChoiceFilter(name='somefield')
+        f = TypedMultipleChoiceFilter(field_name='somefield')
         f.field.choices = ['some', 'values', 'here']
         result = f.filter(qs, [])
         self.assertEqual(qs, result)
@@ -688,7 +632,7 @@ class TypedMultipleChoiceFilterTests(TestCase):
         users = User.objects.all()
 
         for item in filter_list:
-            f = TypedMultipleChoiceFilter(name='favorite_books__pk', conjoined=True)
+            f = TypedMultipleChoiceFilter(field_name='favorite_books__pk', conjoined=True)
             queryset = f.filter(users, item[0])
             expected_pks = [c[0] for c in queryset.values_list('pk')]
             self.assertListEqual(
@@ -730,7 +674,17 @@ class DurationFilterTests(TestCase):
         self.assertIsInstance(field, forms.DurationField)
 
 
-class ModelChoiceFilterTests(TestCase):
+class MockQuerySetMixin:
+    def get_mock_queryset(self):
+        """
+        Configure mock QuerySet with required methods.
+        """
+        qs = mock.NonCallableMock(spec=[])
+        qs.all = mock.Mock(return_value=qs)
+        return qs
+
+
+class ModelChoiceFilterTests(TestCase, MockQuerySetMixin):
 
     def test_default_field_without_queryset(self):
         f = ModelChoiceFilter()
@@ -748,7 +702,7 @@ class ModelChoiceFilterTests(TestCase):
         ])
 
     def test_default_field_with_queryset(self):
-        qs = mock.NonCallableMock(spec=[])
+        qs = self.get_mock_queryset()
         f = ModelChoiceFilter(queryset=qs)
         field = f.field
         self.assertIsInstance(field, forms.ModelChoiceField)
@@ -756,7 +710,7 @@ class ModelChoiceFilterTests(TestCase):
 
     def test_callable_queryset(self):
         request = mock.NonCallableMock(spec=[])
-        qs = mock.NonCallableMock(spec=[])
+        qs = self.get_mock_queryset()
 
         qs_callable = mock.Mock(return_value=qs)
 
@@ -769,7 +723,7 @@ class ModelChoiceFilterTests(TestCase):
 
     def test_get_queryset_override(self):
         request = mock.NonCallableMock(spec=[])
-        qs = mock.NonCallableMock(spec=[])
+        qs = self.get_mock_queryset()
 
         class F(ModelChoiceFilter):
             get_queryset = mock.create_autospec(ModelChoiceFilter.get_queryset, return_value=qs)
@@ -782,7 +736,7 @@ class ModelChoiceFilterTests(TestCase):
         self.assertEqual(field.queryset, qs)
 
 
-class ModelMultipleChoiceFilterTests(TestCase):
+class ModelMultipleChoiceFilterTests(TestCase, MockQuerySetMixin):
 
     def test_default_field_without_queryset(self):
         f = ModelMultipleChoiceFilter()
@@ -799,7 +753,7 @@ class ModelMultipleChoiceFilterTests(TestCase):
         ])
 
     def test_default_field_with_queryset(self):
-        qs = mock.NonCallableMock(spec=[])
+        qs = self.get_mock_queryset()
         f = ModelMultipleChoiceFilter(queryset=qs)
         field = f.field
         self.assertIsInstance(field, forms.ModelMultipleChoiceField)
@@ -807,7 +761,7 @@ class ModelMultipleChoiceFilterTests(TestCase):
 
     def test_filtering_to_field_name(self):
         qs = User.objects.all()
-        f = ModelMultipleChoiceFilter(name='first_name',
+        f = ModelMultipleChoiceFilter(field_name='first_name',
                                       to_field_name='first_name',
                                       queryset=qs)
         user = User.objects.create(first_name='Firstname')
@@ -822,7 +776,7 @@ class ModelMultipleChoiceFilterTests(TestCase):
 
     def test_callable_queryset(self):
         request = mock.NonCallableMock(spec=[])
-        qs = mock.NonCallableMock(spec=[])
+        qs = self.get_mock_queryset()
 
         qs_callable = mock.Mock(return_value=qs)
 
@@ -917,6 +871,27 @@ class NumericRangeFilterTests(TestCase):
         f.filter(qs, value)
         qs.filter.assert_called_once_with(None__endswith=30)
 
+    def test_filtering_distinct(self):
+        f = NumericRangeFilter(distinct=True)
+
+        # range
+        qs = mock.Mock()
+        f.filter(qs, mock.Mock(start=20, stop=30))
+        qs.distinct.assert_called_once()
+        qs.distinct.return_value.filter.assert_called_once_with(None__exact=(20, 30))
+
+        # min
+        qs = mock.Mock()
+        f.filter(qs, mock.Mock(start=20, stop=None))
+        qs.distinct.assert_called_once()
+        qs.distinct.return_value.filter.assert_called_once_with(None__startswith=20)
+
+        # max
+        qs = mock.Mock()
+        f.filter(qs, mock.Mock(start=None, stop=30))
+        qs.distinct.assert_called_once()
+        qs.distinct.return_value.filter.assert_called_once_with(None__endswith=30)
+
 
 class RangeFilterTests(TestCase):
 
@@ -966,13 +941,40 @@ class RangeFilterTests(TestCase):
         f.filter(qs, value)
         qs.filter.assert_called_once_with(None__range=(20, 30))
 
+    def test_filtering_distinct(self):
+        f = RangeFilter(distinct=True)
+
+        # range
+        qs = mock.Mock()
+        f.filter(qs, mock.Mock(start=20, stop=30))
+        qs.distinct.assert_called_once()
+        qs.distinct.return_value.filter.assert_called_once_with(None__range=(20, 30))
+
+        # min
+        qs = mock.Mock()
+        f.filter(qs, mock.Mock(start=20, stop=None))
+        qs.distinct.assert_called_once()
+        qs.distinct.return_value.filter.assert_called_once_with(None__gte=20)
+
+        # max
+        qs = mock.Mock()
+        f.filter(qs, mock.Mock(start=None, stop=30))
+        qs.distinct.assert_called_once()
+        qs.distinct.return_value.filter.assert_called_once_with(None__lte=30)
+
 
 class DateRangeFilterTests(TestCase):
 
     def test_creating(self):
-        f = DateRangeFilter()
-        self.assertIn('choices', f.extra)
-        self.assertEqual(len(DateRangeFilter.options), len(f.extra['choices']))
+        f = DateRangeFilter(empty_label=None)
+        self.assertEqual(len(f.choices), 5)
+        self.assertIs(f.choices, f.extra['choices'])
+
+        f = DateRangeFilter(empty_label=None, choices=[], filters=[])
+        self.assertEqual(f.choices, [])
+        self.assertEqual(f.filters, [])
+        self.assertEqual(len(f.choices), 0)
+        self.assertIs(f.choices, f.extra['choices'])
 
     def test_default_field(self):
         f = DateRangeFilter()
@@ -991,14 +993,29 @@ class DateRangeFilterTests(TestCase):
         qs = mock.Mock(spec=[])
         f = DateRangeFilter()
         with self.assertRaises(AssertionError):
-            f.filter(qs, 999)
+            f.filter(qs, 'tomorrow')
+
+    def test_choices_and_filters_mismatch(self):
+        msg = "Keys must be present in both 'choices' and 'filters'. Missing keys: 'a, b'"
+        with self.assertRaisesMessage(AssertionError, msg):
+            DateRangeFilter(choices=[('a', 'a')], filters={'b': None})
+
+    def test_options_removed(self):
+        msg = "The 'options' attribute has been replaced by 'choices' and 'filters'. " \
+              "See: https://django-filter.readthedocs.io/en/master/guide/migration.html"
+
+        class F(DateRangeFilter):
+            options = None
+
+        with self.assertRaisesMessage(AssertionError, msg):
+            F()
 
     def test_filtering_for_this_year(self):
         qs = mock.Mock(spec=['filter'])
         with mock.patch('django_filters.filters.now') as mock_now:
             now_dt = mock_now.return_value
             f = DateRangeFilter()
-            f.filter(qs, '4')
+            f.filter(qs, 'year')
             qs.filter.assert_called_once_with(
                 None__year=now_dt.year)
 
@@ -1007,7 +1024,7 @@ class DateRangeFilterTests(TestCase):
         with mock.patch('django_filters.filters.now') as mock_now:
             now_dt = mock_now.return_value
             f = DateRangeFilter()
-            f.filter(qs, '3')
+            f.filter(qs, 'month')
             qs.filter.assert_called_once_with(
                 None__year=now_dt.year, None__month=now_dt.month)
 
@@ -1019,7 +1036,7 @@ class DateRangeFilterTests(TestCase):
             mock_d1, mock_d2 = mock.MagicMock(), mock.MagicMock()
             mock_truncate.side_effect = [mock_d1, mock_d2]
             f = DateRangeFilter()
-            f.filter(qs, '2')
+            f.filter(qs, 'week')
             self.assertEqual(
                 mock_td.call_args_list,
                 [mock.call(days=7), mock.call(days=1)]
@@ -1031,7 +1048,7 @@ class DateRangeFilterTests(TestCase):
         with mock.patch('django_filters.filters.now') as mock_now:
             now_dt = mock_now.return_value
             f = DateRangeFilter()
-            f.filter(qs, '1')
+            f.filter(qs, 'today')
             qs.filter.assert_called_once_with(
                 None__year=now_dt.year,
                 None__month=now_dt.month,
@@ -1042,10 +1059,10 @@ class DateRangeFilterTests(TestCase):
         with mock.patch('django_filters.filters.now') as mock_now:
             now_dt = mock_now.return_value
             f = DateRangeFilter()
-            f.filter(qs, '5')
+            f.filter(qs, 'yesterday')
             qs.filter.assert_called_once_with(
-                None__year=now_dt.year,
-                None__month=now_dt.month,
+                None__year=(now_dt - timedelta(days=1)).year,
+                None__month=(now_dt - timedelta(days=1)).month,
                 None__day=(now_dt - timedelta(days=1)).day,
             )
 
@@ -1140,6 +1157,52 @@ class DateTimeFromToRangeFilterTests(TestCase):
             None__range=(datetime(2015, 4, 7, 8, 30), datetime(2015, 9, 6, 11, 45)))
 
 
+class IsoDateTimeFromToRangeFilterTests(TestCase):
+
+    def test_default_field(self):
+        f = IsoDateTimeFromToRangeFilter()
+        field = f.field
+        self.assertIsInstance(field, IsoDateTimeRangeField)
+
+    def test_filtering_range(self):
+        qs = mock.Mock(spec=['filter'])
+        value = mock.Mock(
+            start=datetime(2015, 4, 7, 8, 30), stop=datetime(2015, 9, 6, 11, 45))
+        f = IsoDateTimeFromToRangeFilter()
+        f.filter(qs, value)
+        qs.filter.assert_called_once_with(
+            None__range=(datetime(2015, 4, 7, 8, 30), datetime(2015, 9, 6, 11, 45)))
+
+    def test_filtering_start(self):
+        qs = mock.Mock(spec=['filter'])
+        value = mock.Mock(start=datetime(2015, 4, 7, 8, 30), stop=None)
+        f = IsoDateTimeFromToRangeFilter()
+        f.filter(qs, value)
+        qs.filter.assert_called_once_with(None__gte=datetime(2015, 4, 7, 8, 30))
+
+    def test_filtering_stop(self):
+        qs = mock.Mock(spec=['filter'])
+        value = mock.Mock(start=None, stop=datetime(2015, 9, 6, 11, 45))
+        f = IsoDateTimeFromToRangeFilter()
+        f.filter(qs, value)
+        qs.filter.assert_called_once_with(None__lte=datetime(2015, 9, 6, 11, 45))
+
+    def test_filtering_skipped_with_none_value(self):
+        qs = mock.Mock(spec=['filter'])
+        f = IsoDateTimeFromToRangeFilter()
+        result = f.filter(qs, None)
+        self.assertEqual(qs, result)
+
+    def test_filtering_ignores_lookup_expr(self):
+        qs = mock.Mock()
+        value = mock.Mock(
+            start=datetime(2015, 4, 7, 8, 30), stop=datetime(2015, 9, 6, 11, 45))
+        f = IsoDateTimeFromToRangeFilter(lookup_expr='gte')
+        f.filter(qs, value)
+        qs.filter.assert_called_once_with(
+            None__range=(datetime(2015, 4, 7, 8, 30), datetime(2015, 9, 6, 11, 45)))
+
+
 class TimeRangeFilterTests(TestCase):
 
     def test_default_field(self):
@@ -1203,7 +1266,7 @@ class AllValuesFilterTests(TestCase):
         self.assertIsInstance(field, forms.ChoiceField)
 
     def test_empty_value_in_choices(self):
-        f = AllValuesFilter(name='username')
+        f = AllValuesFilter(field_name='username')
         f.model = User
 
         self.assertEqual(list(f.field.choices), [
@@ -1211,48 +1274,79 @@ class AllValuesFilterTests(TestCase):
         ])
 
 
-class LookupTypesTests(TestCase):
-    def test_custom_lookup_exprs(self):
-        filters.LOOKUP_TYPES = [
+class LookupChoiceFilterTests(TestCase):
+
+    def test_normalize_lookup_no_display_label(self):
+        # display label has underscores replaced and is capitalized
+        display_label = LookupChoiceFilter.normalize_lookup('has_key')
+        self.assertEqual(display_label, ('has_key', 'Has key'))
+
+    def test_normalize_lookup_with_display_label(self):
+        # display label is not transformed if provided
+        display_label = LookupChoiceFilter.normalize_lookup(('equal', 'equals'))
+        self.assertEqual(display_label, ('equal', 'equals'))
+
+    def test_lookup_choices_default(self):
+        # Lookup choices should default to the model field's registered lookups
+        f = LookupChoiceFilter(field_name='username', lookup_choices=None)
+        f.model = User
+
+        choice_field = f.field.fields[1]
+        self.assertEqual(
+            len(choice_field.choices),
+            len(User._meta.get_field('username').get_lookups()) + 1
+        )
+
+        field_choices = dict(choice_field.choices)
+        self.assertEqual(field_choices['exact'], 'Exact')
+        self.assertEqual(field_choices['startswith'], 'Startswith')
+
+    def test_lookup_choices_list(self):
+        f = LookupChoiceFilter(field_name='username', lookup_choices=[
+            'exact',
+            'startswith',
+            'has_key'
+        ])
+
+        choice_field = f.field.fields[1]
+        self.assertEqual(list(choice_field.choices), [
+            ('', '---------'),
+            ('exact', 'Exact'),
+            ('startswith', 'Startswith'),
+            ('has_key', 'Has key'),
+        ])
+
+    def test_lookup_choices_pairs(self):
+        f = LookupChoiceFilter(field_name='username', lookup_choices=[
+            ('exact', 'Is equal to'),
+            ('startswith', 'Starts with'),
+        ])
+
+        choice_field = f.field.fields[1]
+        self.assertEqual(list(choice_field.choices), [
             ('', '---------'),
             ('exact', 'Is equal to'),
-            ('not_exact', 'Is not equal to'),
-            ('lt', 'Lesser than'),
-            ('gt', 'Greater than'),
-            ('gte', 'Greater than or equal to'),
-            ('lte', 'Lesser than or equal to'),
             ('startswith', 'Starts with'),
-            ('endswith', 'Ends with'),
-            ('contains', 'Contains'),
-            ('not_contains', 'Does not contain'),
-        ]
+        ])
 
-        f = Filter(lookup_expr=None)
-        field = f.field
-        choice_field = field.fields[1]
-        all_choices = choice_field.choices
+    def test_lookup_choices_empty_label_default(self):
+        f = LookupChoiceFilter(field_name='username', lookup_choices=[])
 
-        self.assertIsInstance(field, LookupTypeField)
-        self.assertEqual(all_choices, filters.LOOKUP_TYPES)
-        self.assertEqual(all_choices[1][0], 'exact')
-        self.assertEqual(all_choices[1][1], 'Is equal to')
+        choice_field = f.field.fields[1]
+        self.assertEqual(list(choice_field.choices), [('', '---------')])
 
-        custom_f = Filter(lookup_expr=('endswith', 'not_contains'))
-        custom_field = custom_f.field
-        custom_choice_field = custom_field.fields[1]
-        my_custom_choices = custom_choice_field.choices
+    def test_lookup_choices_empty_label_disabled(self):
+        f = LookupChoiceFilter(field_name='username', empty_label=None, lookup_choices=[])
 
-        available_lookup_exprs = [
-            ('endswith', 'Ends with'),
-            ('not_contains', 'Does not contain'),
-        ]
+        choice_field = f.field.fields[1]
+        self.assertEqual(list(choice_field.choices), [])
 
-        self.assertIsInstance(custom_field, LookupTypeField)
-        self.assertEqual(my_custom_choices, available_lookup_exprs)
-        self.assertEqual(my_custom_choices[0][0], 'endswith')
-        self.assertEqual(my_custom_choices[0][1], 'Ends with')
-        self.assertEqual(my_custom_choices[1][0], 'not_contains')
-        self.assertEqual(my_custom_choices[1][1], 'Does not contain')
+    def test_filtering(self):
+        qs = mock.Mock(spec=['filter'])
+        f = LookupChoiceFilter(field_name='somefield', lookup_choices=['some_lookup_expr'])
+        result = f.filter(qs, Lookup('value', 'some_lookup_expr'))
+        qs.filter.assert_called_once_with(somefield__some_lookup_expr='value')
+        self.assertNotEqual(qs, result)
 
 
 class CSVFilterTests(TestCase):
@@ -1410,7 +1504,7 @@ class OrderingFilterTests(TestCase):
 
     def test_normalize_fields(self):
         f = OrderingFilter.normalize_fields
-        O = OrderedDict
+        O = OrderedDict  # noqa
 
         self.assertIn('a', f({'a': 'b'}))
 
