@@ -1,5 +1,3 @@
-from __future__ import absolute_import, unicode_literals
-
 from django import forms
 from django.test import TestCase, override_settings
 
@@ -81,7 +79,7 @@ class FilterSetFormTests(TestCase):
     def test_complex_form_fields(self):
         class F(FilterSet):
             username = CharFilter(label='Filter for users with username')
-            exclude_username = CharFilter(name='username', lookup_expr='iexact', exclude=True)
+            exclude_username = CharFilter(field_name='username', lookup_expr='iexact', exclude=True)
 
             class Meta:
                 model = User
@@ -135,7 +133,7 @@ class FilterSetFormTests(TestCase):
 
     def test_form_field_with_manual_name(self):
         class F(FilterSet):
-            book_title = CharFilter(name='title')
+            book_title = CharFilter(field_name='title')
 
             class Meta:
                 model = Book
@@ -147,7 +145,7 @@ class FilterSetFormTests(TestCase):
 
     def test_form_field_with_manual_name_and_label(self):
         class F(FilterSet):
-            f1 = CharFilter(name='title', label="Book title")
+            f1 = CharFilter(field_name='title', label="Book title")
 
             class Meta:
                 model = Book
@@ -225,3 +223,35 @@ class FilterSetFormTests(TestCase):
                 F().form.fields['title__in'].help_text,
                 ''
             )
+
+
+class FilterSetValidityTests(TestCase):
+
+    class F(FilterSet):
+        class Meta:
+            model = Book
+            fields = ['title', 'price']
+
+    def test_not_bound(self):
+        f = self.F()
+
+        self.assertFalse(f.is_bound)
+        self.assertFalse(f.is_valid())
+        self.assertEqual(f.data, {})
+        self.assertEqual(f.errors, {})
+
+    def test_is_bound_and_valid(self):
+        f = self.F({'title': 'Some book'})
+
+        self.assertTrue(f.is_bound)
+        self.assertTrue(f.is_valid())
+        self.assertEqual(f.data, {'title': 'Some book'})
+        self.assertEqual(f.errors, {})
+
+    def test_is_bound_and_not_valid(self):
+        f = self.F({'price': 'four dollars'})
+
+        self.assertTrue(f.is_bound)
+        self.assertFalse(f.is_valid())
+        self.assertEqual(f.data, {'price': 'four dollars'})
+        self.assertEqual(f.errors, {'price': ['Enter a number.']})
